# -*- coding: utf-8 -*-
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gdk

import cairo
from threading import Thread
from time import sleep

COLORS = {1:(204/255, 0, 0), 2:(0, 153/255, 51/255), 3:(51/255, 102/255, 255/255), 4:(1, 1, 1), 5:(255/255, 255/255, 102/255), 6:(153/255, 51/255, 153/255)}

class Jewels(Gtk.Window):
    playing_area = []
    hidden_stones = []
    active_stone = None
    score = None
    time = None
    playing = False
    
    def __init__(self):
        Gtk.Window.__init__(self, title="Hra Jewels")
        #self.set_size_request(640, 440)
        
        self.box = Gtk.Box()
        self.add(self.box)
        
        # Vykreslovací oblast
        self.darea = Gtk.DrawingArea()
        self.darea.set_size_request(500, 500)
        self.darea.connect("draw", self.on_draw)
        self.darea.set_events(Gdk.EventMask.BUTTON_PRESS_MASK)
        
        self.darea.set_app_paintable(True)
        self.box.pack_start(self.darea, True, True, 0)
        
        # Pravý sloupec
        menu = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.box.pack_start(menu, False, False, 10)

        # Načtení souboru
        txt_filter = Gtk.FileFilter()
        txt_filter.set_name("Textové soubory")
        txt_filter.add_pattern("*.txt")
        
        all_filter = Gtk.FileFilter()
        all_filter.set_name("Všechny soubory")
        all_filter.add_pattern("*")
        
        self.file_chooser = Gtk.FileChooserButton("Načíst hrací plochu")
        self.file_chooser.add_filter(txt_filter)
        self.file_chooser.add_filter(all_filter)
        self.file_chooser.connect("file-set", self.on_file_set)
        menu.pack_start(self.file_chooser, False, True, 0)
        
        # Startovací tlačítko
        self.start_button = Gtk.Button(label="Start")
        self.start_button.set_sensitive(False)
        self.start_button.connect("clicked", self.play)
        menu.pack_start(self.start_button, False, True, 0)
        
        # Body
        self.score_label = Gtk.Label("Celkem bodů: " + str(self.score))
        menu.pack_end(self.score_label, False, True, 10)
        
        # Čas
        self.time_label = Gtk.Label("5:00")
        menu.pack_end(self.time_label, False, True, 10)
        
    def on_file_set(self, widget):
        self.start_button.set_sensitive(True)
        
    def play(self, widget):
        self.playing = True
        self.playing_area.clear()
        self.hidden_stones.clear()
        self.time = 300
        self.score = 0
        
        self.darea.connect("button-press-event", self.on_button_press)
        
        f = open(self.file_chooser.get_filename()).read()
        f = f.split("\n")
        for row in f[:8]:
            self.playing_area.append([int(stone) for stone in row])
        for row in f[8:]:
            self.hidden_stones.append([int(stone) for stone in row])
        self.darea.queue_draw()
        
        self.interval = Thread(target=self.time_callback)
        self.interval.start()
        
    def time_callback(self):
        while self.time > 0:
            self.time -= 1
            self.time_label.set_text("%d:%d" % (self.time // 60, self.time % 60))
            sleep(1)
        self.playing = False
            
        print("Čas vypršel!")
        
        
    def on_draw(self, widget, cr):
        if len(self.playing_area) > 0:
            for x in range(8):
                if len(self.hidden_stones[x]) > 0:
                    stone = self.hidden_stones[x][0]
                    cr.set_source_rgb(*COLORS[stone])
                    cr.arc(50 + x*50, 50, 20, 0, 2*3.14159)
                    cr.fill()
                        
            cr.set_source_rgb(0, 0, 0)
            cr.move_to(25, 80)
            cr.line_to(25 + 8*50, 80)
            cr.stroke()                        
                        
            for y, row in enumerate(self.playing_area):
                for x, stone in enumerate(row):
                    if stone != 0:
                        if (x, y) == self.active_stone:
                            cr.set_source_rgb(255/255, 153/255, 51/255)
                            cr.arc(50 + x*50, 60 + 50 + y*50, 21, 0, 2*3.14159)
                            cr.stroke()
                        cr.set_source_rgb(*COLORS[stone])
                        cr.arc(50 + x*50, 60 + 50 + y*50, 20, 0, 2*3.14159)
                        cr.fill()
                    
    def find_sequences(self, area):
        score = 0
        sequences = set()
        
        # Sled v řádku
        for y, row in enumerate(area):
            for x in range(len(row)-2):
                if row[x] == row[x+1] == row[x+2]:
                    sequences.add((x,y))
                    sequences.add((x+1,y))
                    sequences.add((x+2,y))
                    score += 3
                    
        # Sled ve sloupci
        for y in range(len(area)-2):
            for x in range(8):
                if area[y][x] == area[y+1][x] == area[y+2][x]:
                    sequences.add((x,y))
                    sequences.add((x,y+1))
                    sequences.add((x,y+2))
                    score += 3
                    
        return sequences, score
                    
    def on_button_press(self, widget, event):
        if not self.playing:
            return
            
        if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 1:
            x = int((event.x-25) / 50)
            y = int((event.y-25-60) / 50)
            
            if y < 0:
                return
            
            if self.active_stone == None:
                self.active_stone = (x, y)
            else:
                if abs(self.active_stone[0] - x) == 1 and self.active_stone[1] == y:
                    self.change(x, y)
                    self.active_stone = None
                elif abs(self.active_stone[1] - y) == 1 and self.active_stone[0] == x:
                    self.change(x, y)
                    self.active_stone = None
                else:
                    self.active_stone = (x, y)
                    
            self.darea.queue_draw()
            
    def change(self, *stone1):
        stone2 = self.active_stone
        
        # Otočení kamenů
        area = [row.copy() for row in self.playing_area]
        area[stone1[1]][stone1[0]], area[stone2[1]][stone2[0]] = area[stone2[1]][stone2[0]], area[stone1[1]][stone1[0]]
        
        # Dokuď budou ve hře nějaké sledy
        sequences = True
        while sequences:
            result = self.find_sequences(area)
            if result[1] > 0:
                self.score += result[1]
                self.score_label.set_text("Celkem bodů: " + str(self.score))
                for x, y in result[0]:
                    area[y][x] = 0
                
                # Padání kamenů
                for x in range(8):
                    for y in range(8)[::-1]:
                        if area[y][x] == 0:
                            for n in range(y)[::-1]:
                                if area[n][x] != 0:
                                    area[y][x] = area[n][x]
                                    area[n][x] = 0
                                    break
                            else:
                                for i in range(y+1)[::-1]:
                                    if len(self.hidden_stones[x]) > 0:
                                        area[i][x] = self.hidden_stones[x].pop(0)
                self.playing_area = area.copy()
            else:
                sequences = False
                
app = Jewels()
app.connect("delete-event", Gtk.main_quit)
app.show_all()
Gtk.main()